//  Sky.cc - Helper class for Caelum.
//
//  Copyright (C) 2003 Sam Varner
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "Sky.h"

#include <cassert>

// Windows puts OpenGL 1.2 and 1.3 stuff in glext.h.
#if HAVE_GL_GLEXT_H
# include <GL/glext.h>
#endif
#include <GL/glut.h>

#include <vamos/geometry/Constants.h>

Sky::Sky (std::string image, int width, int height) : 
  m_width (width),
  m_height (height)
{
  // Load the image.
  mp_image = new Vamos_Geometry::Gl_Texture_Image (image);

  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

  // Repeat in the east-west direction.
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  // Clamp in the north-south direction.
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
}

void
Sky::key_press (unsigned char key, int, int)
{
  switch (key)
	{
	case 'q': // fall through
	case 27:  // escape
	  std::exit (0);
	  break;
	}
}

void
Sky::display ()
{
  // Clear the entire window.
  glViewport (0, 0, m_width * 4, m_height * 3);
  glClear (GL_COLOR_BUFFER_BIT);

  // Enable sub-windows.
  glEnable (GL_SCISSOR_TEST);

  // Draw the front view.
  int x = 0;
  int y = m_height;
  glViewport (x, y, m_width, m_height);
  glScissor (x, y, m_width, m_height);

  // Set the camera.
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (90, 1.0, 0.1, 1.1);
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  gluLookAt (0.0, 0.0, 0.0, // camera position 
			 0.1, 0.0, 0.0,  // target 
			 0.0, 0.0, 0.1); // up direction
  draw ();

  // Draw the right view.
  x += m_width;
  glViewport (x, y, m_width, m_height);
  glScissor (x, y, m_width, m_height);
  glRotated (90.0, 0.0, 0.0, 1.0);
  draw ();

  // Draw the back view.
  x += m_width;
  glViewport (x, y, m_width, m_height);
  glScissor (x, y, m_width, m_height);
  glRotated (90.0, 0.0, 0.0, 1.0);
  draw ();

  // Draw the left view.
  x += m_width;
  glViewport (x, y, m_width, m_height);
  glScissor (x, y, m_width, m_height);
  glRotated (90.0, 0.0, 0.0, 1.0);
  draw ();

  // Draw the up view.
  x = 0;
  y = 0;
  glViewport (x, y, m_width, m_height);
  glScissor (x, y, m_width, m_height);
  glRotated (90.0, 0.0, 0.0, 1.0);
  glRotated (90.0, 0.0, -1.0, 0.0);
  draw ();

  // Draw the down view.
  y = m_height*2;
  glViewport (x, y, m_width, m_height);
  glScissor (x, y, m_width, m_height);
  glRotated (180.0, 0.0, -1.0, 0.0);
  draw ();

  glFlush ();
  glutSwapBuffers ();
}
