#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/extensions/TestFactoryRegistry.h>

#include "../include/vamos/geometry/Three_Vector.h"

using namespace Vamos_Geometry;

class Null_Vector : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE (Null_Vector);
  CPPUNIT_TEST (Zero_Length);
  CPPUNIT_TEST_SUITE_END ();

  Three_Vector m_Vector;

public:
  void setUp () {}
  void tearDown () {}

  void Zero_Length () {
    CPPUNIT_ASSERT (m_Vector.abs () == 0.0);
  }
};

class Unit_Vectors : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE (Unit_Vectors);
  CPPUNIT_TEST (dot_products);
  CPPUNIT_TEST (cross_products);
  CPPUNIT_TEST (add);
  CPPUNIT_TEST (subtract);
  CPPUNIT_TEST_SUITE_END ();

  Three_Vector* mp_x;
  Three_Vector* mp_y;
  Three_Vector* mp_z;

public:
  void setUp () {
    mp_x = new Three_Vector (1.0, 0.0, 0.0);
    mp_y = new Three_Vector (0.0, 1.0, 0.0);
    mp_z = new Three_Vector (0.0, 0.0, 1.0);
  }
  void tearDown () {
    delete mp_x;
    delete mp_y;
    delete mp_z;
  }

  void dot_products () {
    CPPUNIT_ASSERT (mp_x->dot (*mp_y) == 0.0); 
    CPPUNIT_ASSERT (mp_x->dot (*mp_z) == 0.0); 
    CPPUNIT_ASSERT (mp_y->dot (*mp_x) == 0.0); 
    CPPUNIT_ASSERT (mp_y->dot (*mp_z) == 0.0); 
    CPPUNIT_ASSERT (mp_z->dot (*mp_x) == 0.0); 
    CPPUNIT_ASSERT (mp_z->dot (*mp_y) == 0.0); 
  }
  void cross_products () {
    CPPUNIT_ASSERT (mp_x->cross (*mp_y) == *mp_z);
    CPPUNIT_ASSERT (mp_x->cross (*mp_z) == -*mp_y);
    CPPUNIT_ASSERT (mp_y->cross (*mp_x) == -*mp_z);
    CPPUNIT_ASSERT (mp_y->cross (*mp_z) == *mp_x);
    CPPUNIT_ASSERT (mp_z->cross (*mp_x) == *mp_y);
    CPPUNIT_ASSERT (mp_z->cross (*mp_y) == -*mp_x);
  }
  void add () {
    CPPUNIT_ASSERT (*mp_x + *mp_y + *mp_z == Three_Vector (1.0, 1.0, 1.0));
  }
  void subtract () {
    CPPUNIT_ASSERT (Three_Vector (1.0, 1.0, 1.0) - *mp_x - *mp_y == *mp_z);
  }
};

class One_Two_Three : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (One_Two_Three);
  CPPUNIT_TEST (Negate);
  CPPUNIT_TEST_SUITE_END ();

  Three_Vector* m_123;

public:
  void setUp () {
    m_123 = new Three_Vector (1.0, 2.0, 3.0);
  }
  void tearDown ()
  {
    delete m_123;
  }

  void Negate ()
  {
    CPPUNIT_ASSERT (-*m_123 == Three_Vector (-1.0, -2.0, -3.0));
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION (Null_Vector);
CPPUNIT_TEST_SUITE_REGISTRATION (Unit_Vectors);
CPPUNIT_TEST_SUITE_REGISTRATION (One_Two_Three);
int main ()
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest (CppUnit::TestFactoryRegistry::getRegistry ().makeTest ());
  return !runner.run ();
}
