#include <iostream>
#include <algorithm>
#include <cmath>

// A magic formula for Fx and Fy.
double
magic (double sigma)
{
  return -4.0 * sin (1.65 * atan (0.4 * sigma - 10.0 * atan (0.4 * sigma)));
}

// No interaction between Fx and Fy
void
fric1 (double sigma, double alpha, double* Fx, double* Fy)
{
  *Fx = magic (sigma);
  *Fy = magic (alpha);
}

// Interaction based on the magnitude of the slip
void
fric2 (double sigma, double alpha, double* Fx, double* Fy)
{
  double sigma_x_tot = -sigma / (1.0 + std::abs (sigma));
  double sigma_y_tot = tan (alpha * M_PI / 180.0)
	/ (1.0 + std::abs (sigma));
  double sigma_tot = std::sqrt (sigma_x_tot * sigma_x_tot
								+ sigma_y_tot * sigma_y_tot);

  *Fx = sigma_x_tot / sigma_tot * magic (sigma_tot);
  *Fy = sigma_y_tot / sigma_tot * magic (sigma_tot);
}

// Genta's last model in section 2.7.  I can't find a way to get
// sensible cuves out of this one.
void
fric3 (double sigma, double alpha, double* Fx, double* Fy)
{
  double sigma_x_tot = -sigma / (1.0 + std::abs (sigma));
  double sigma_y_tot = tan (alpha * M_PI / 180.0)
	/ (1.0 + std::abs (sigma));
  
  const double sigma_x_max = 0.4;
  const double sigma_y_max = 0.4;
  
  double sigma_star_x = sigma_x_tot / sigma_x_max;
  double sigma_star_y = sigma_y_tot / sigma_y_max;
  double sigma_star = std::sqrt (sigma_star_x * sigma_star_x 
								 + sigma_star_y * sigma_star_y);
  
  double factor = std::min (sigma_star, 1.0);
  double x_ratio = sigma_star_x / sigma_star;
  double y_ratio = sigma_star_y / sigma_star;
  
  double Fx0 = magic (sigma_star_x);
  double Fy0 = magic (sigma_star_y);
  
  double F_star_x0 = Fx0 - factor * (Fx0 - Fy0) * y_ratio * y_ratio;
  double F_star_y0 = Fy0 - factor * (Fy0 - Fx0) * x_ratio * x_ratio;
  
  *Fx = x_ratio * F_star_x0;
  *Fy = y_ratio * F_star_y0;
}

// Print out a column of Fx and Fy for various values of slip angle
// and slip ratio.  You can redirect the output to a file with 
// "fric > out" and veiw the curves in gnuplot with "plot 'out'".  
int
main ()
{
  double alpha_max = 20.0;
  double alpha_inc = 5.0;
  double alpha = -alpha_max;

  while (alpha <= alpha_max)
	{
	  double sigma_max = 2.0;
	  double sigma_inc = 0.01;
	  double sigma = -sigma_max;
	  while (sigma <= sigma_max)
		{
		  // std::cerr << alpha << ' ' << sigma << std::endl;

		  double Fx, Fy;

		  // Pick one model and comment out the rest.
		  // fric1 (sigma, alpha * M_PI / 180.0, &Fx, &Fy);
		  fric2 (sigma, alpha * M_PI / 180.0, &Fx, &Fy);
		  // fric3 (sigma, alpha * M_PI / 180.0, &Fx, &Fy);
		  std::cout << Fx << ' ' << Fy << std::endl; 

		  sigma += sigma_inc;
		}
	  alpha += alpha_inc;
	}

  return 0;
}
