//	Vamos Automotive Simulator
//  Copyright (C) 2001--2002 Sam Varner
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <vamos/geometry/Three_Vector.h>

#include <iostream>

using Vamos_Geometry::Three_Vector;

int
main ()
{
  Three_Vector vec1;
  std::cout << "Three_Vector vec1;\n\t" << vec1 << std::endl;

  Three_Vector vec2 (1.0, 2.0, 3.0);
  std::cout << "Three_Vector vec2 (1.0, 2.0, 3.0);\n\t" << vec2 << std::endl;

  double c_vec [3] = { 4.0, 5.0, 6.0 };
  Three_Vector vec3 (c_vec);
  std::cout << "double c_vec [3] = { 4.0, 5.0, 6.0 };\n" 
	   << "Three_Vector vec3 (c_vec);\n\t" << vec3 << std::endl;

  Three_Vector vec4 (vec3);
  std::cout << "Three_Vector vec4 (vec3);\n\t" << vec4 << std::endl;

  vec4 = vec2;
  std::cout << "vec4 = vec2;\n\t" << vec4 << std::endl;

  std::cout << std::endl;
  std::cout << "subscript return\n";
  std::cout << "vec2 [0] = " << vec2 [0] << std::endl
	   << "vec2 [1] = " << vec2 [1] << std::endl
	   << "vec2 [2] = " << vec2 [2] << std::endl;
  std::cout << "subscript assignment\n"; 
  vec1 [0] = 3.0;
  std::cout << "vec1 [0] = 3.0;\n";
  vec1 [1] = 4.0;
  std::cout << "vec1 [1] = 3.0;\n";
  vec1 [2] = 5.0;
  std::cout << "vec1 [2] = 3.0;\n";
  std::cout << "vec1\n\t" << vec1 << std::endl;

  vec1.zero ();
  std::cout << "vec1.zero ();\n\t" << vec1 << std::endl;
  std::cout << "vec2.abs ();\n\t" << vec2.abs() << std::endl;
  std::cout << "vec2.dot (vec3);\n\t" << vec2.dot (vec3) << std::endl;
  vec4 = vec2.cross (vec3);
  std::cout << "vec4 = vec2.cross (vec3);\n\t" << vec4 << std::endl;
  std::cout << "Check orthogonality.\n"; 
  std::cout << "vec2.dot (vec4);\n\t" << vec2.dot (vec4) << std::endl;
  std::cout << "vec3.dot (vec4);\n\t" << vec3.dot (vec4) << std::endl;

  vec4 = vec2.project (vec3);
  std::cout << "vec4 = vec2.project (vec3);\n\t" << vec4 << std::endl;
  std::cout << "vec4 / vec3\n\t"
	   << vec4 [0] / vec3 [0] << " "
	   << vec4 [1] / vec3 [1] << " "
	   << vec4 [2] / vec3 [2] << std::endl;
  vec4 = vec2.unit ();
  std::cout << "vec2 / vec4\n\t"
	   << vec2 [0] / vec4 [0] << " "
	   << vec2 [1] / vec4 [1] << " "
	   << vec2 [2] / vec4 [2] << std::endl;
  std::cout << "vec4 = vec2.unit ();\n\t" << vec4 << std::endl;
  std::cout << "vec2.unit ().abs ();\n\t" << vec2.unit ().abs () << std::endl;

  vec1 = Three_Vector (1.0, 0.0, 1.0);
  Three_Vector point1 (1.0, 1.0, 1.0);
  Three_Vector point2 (1.0, 1.0, 2.0);
  double dist = vec1.perp_distance (point1, point2);
  std::cout << "vec1.perp_distance (point1, point2) = " << dist << std::endl;

  std::cout << "Math operations\n";
  vec1 += vec2;
  std::cout << vec1;
  std::cout << std::endl;

  return 0;
}
